#include <stdlib.h>
#include <ctype.h>
#include <tcl.h>
#include <stdio.h>
#include <math.h>
#include <string.h>		/* strcat */

#include "rpm-internal.h"
#include <stdio.h>
#include <stdlib.h>

static void smooth3r(float* x, float* y, int n);
static int big3r( float* x, float* y, int n);
static void r3( float* x, float* y, int n);
static void endpt( float* x, float* y, int n);
static float fm3( float x, float y, float z);
static int stable ( float* x, float* y, int n);
static void swap( float* x, float* y, int n);
static void peaks( float* x, float* y, int n);
static void hann( float* x, float* y, int n);
static float erule( float x, float y, float z);

static char rcsid[] = "$Id: rpmSmooth.c,v 1.2 1999/08/09 13:51:11 martin Exp $";

static void smooth3r(float* x, float* y, int n)
{

  if( n < 3 )
    swap( x, y, n );
  else {
    big3r( x, y, n );
    peaks( x, y, n );
    swap( x, y, n );
    big3r( x, y, n );
    peaks( x, y, n );
    swap( x, y, n );
    big3r( x, y, n );
    hann( x, y, n );
  }
}

static int big3r( float* x, float* y, int n)
{
  int passes;

  passes = 0;
  r3(x, y, n);
  while(stable(x, y, n) <= 0 ) {
    passes = passes + 1;
    swap( x, y, n );
    r3( x, y, n );
  }
  endpt( x, y, n);
  return passes;
}

static void r3( float* x, float* y, int n)
{
  int i;

  for(i = 2; i <= n-1; i++)
    y[i] = fm3( x[i-1], x[i], x[i+1]);
  y[1] = x[1];
  y[n] = x[n];
}

static void endpt(float* x, float* y, int n)
{
  x[1] = erule( x[1], x[2], x[3]);
  x[n] = erule( x[n], x[n-1], x[n-2]);
}

static float fm3(float x, float y, float z)
{
  float a[4], t;
  int i, j;

  a[1] = x;
  a[2] = y;
  a[3] = z;
  for(i = 1; i <= 2; i++)
    for(j = 1; j <= (3 - i); j++)
      if( a[j] > a[j+1] ) {
	t = a[j];
	a[j] = a[j+1];
	a[j+1] = t;
      }

  return a[2];
}

static int stable(float* x, float* y, int n)
{
  int i;
  for(i = 1; i <= n; i++)
    if( x[i] != y[i] )
      return 0;
  return 1;
}

static void swap( float* x, float* y, int n)
{
  int i;
  float t;

  for(i = 1; i <= n; i++) {
    t = x[i];
    x[i] = y[i];
    y[i] = t;
  }
}

static void peaks( float* x, float* y, int n)
{
  int i;

  for(i = 1; i <= n; i++)
    y[i] = x[i];
  if( n < 4 )
    return;
  for(i = 2; i <= n-2; i++)
    if( x[i] != x[i+1] )
      ;
    else if( x[i] == x[i-1] )
      ;
    else if( x[i+1] == x[i+2] )
      ;
    else if( (x[i-1] > x[i]) && (x[i+1] > x[i+2]) )
      ;
    else if( (x[i-1] < x[i]) && (x[i+1] < x[i+2]) )
      ;
    else {
      if( i > 2 )
	y[i] = erule( x[i], x[i-1], x[i-2] );
      else
	y[i] = x[i-1];
      if( i < n-2 )
	y[i+1] = erule( x[i+1], x[i+2], x[i+3] );
      else
	y[i+1] = y[i+2];
    }
}

static void hann( float* x, float* y, int n)
{	
	int i;

	for(i = 2; i <= n - 1; i++)
	  y[i] = 0.25 * ( x[i-1] + 2.0*x[i] + x[i+1] );
	y[1] = x[1];
	y[n] = x[n];
}

static float erule( float x, float y, float z)
{
	return fm3( x, y, y-2.0*(z-y));
}
/*
**    list 3rssh
*/
int rpm3rsshSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    rpmBinary rlist, retv;
    int n;

    rlist = rpmPop();
    n = rpmLength(rlist);
    if(n == 0)
	return TCL_OK;

    retv = rpmNewBinaryD1(n);
    smooth3r(&rlist.v[0] - 1, &retv.v[0] - 1, n);
    rpmFreeBinary(rlist);
    rpmPushB(retv);
    return TCL_OK;
}

/*
**	list count runavg
*/
int rpmRunavgSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    int decim;
    int i;
    int n;
    rpmBinary lin, lout;
    double q;
    int il;
    int ir;
    int j;

    decim = rpmPopScalar();
    lin = rpmPop();
    n = rpmLength(lin);
    if(decim <= 1)
	decim = 1;
    lout = rpmNewBinaryD1(n);
    for(i = 0; i < n; i++) {
      il = i - decim / 2;
      ir = il + decim - 1;
      if(il < 0) il = 0;
      if(ir >= n) ir = n - 1;
      q = 0.0;
      for(j = il; j <= ir; j++)
	q += lin.v[j];
      lout.v[i] = q / (ir - il + 1);
    }
    rpmFreeBinary(lin);
    rpmPushB(lout);
    return TCL_OK;
}
